# -*- encoding: utf-8 -*-
"""
These module provides support for:

* creating new derived :class:`GObj` class with your
  :term:`simple-machine`.
* creating :term:`gobj`'s with :func:`create_gobj` factory function.

The :class:`GObj` class provides support for:

* sending events:

    * sending events by direct delivery: :meth:`GObj.send_event`.
    * sending events by queues: :meth:`GObj.post_event`.
    * sending events to subscribers: :meth:`GObj.broadcast_event`.

* receiving events:

    * directly from another :term:`gobj`'s who knows you.
    * subscribing to events by the :meth:`GObj.subscribe_event` method.
    * you can filtering events being broadcasting with
      :meth:`GObj.set_owned_event_filter` method.

.. autoclass:: Event
    :members:

.. autoclass:: GObj
    :members: start_up, create_gobj, destroy_gobj,
        send_event, post_event, broadcast_event,
        subscribe_event, delete_subscription, set_owned_event_filter,
        search_unique_gobj,
        overwrite_parameters, overwrite_few_parameters

    .. attribute:: name

        My name.

        Set by :meth:`create_gobj`.

    .. attribute:: parent

        My parent, destination of my events... or not.

        Set by :meth:`create_gobj`.

    .. attribute:: dl_childs

        Set of my gobj childs. Me too like be parent!.


    .. method:: set_new_state

        Set a new state.
        Method to used inside actions, to force the change of state.

        :param new_state: new state to set.

        ``new_state`` must match some of the state names of the
        machine's :term:`state-list` or a :exc:`StateError` exception
        will be raised.

    .. method:: get_current_state

        Return the name of the current state.

        If there is no state it returns ``None``.

    .. method:: get_input_event_list

       Return the list with the :term:`input-event`'s names.

    .. method:: get_output_event_list

       Return the list with the :term:`output-event`'s names.


.. autoexception:: ParentError

.. autoexception:: DestinationError

.. autoexception:: GObjError

.. autoexception:: EventError

.. autoexception:: StateError

.. autoexception:: MachineError

.. autoexception:: EventNotAcceptedError

.. autoexception:: QueueError

"""
import threading
import logging
import ginsfsm.globals  # made it import available

from ginsfsm.compat import string_types

from ginsfsm.smachine import (
    SMachine,
    EventError,
    EventNotAcceptedError,  # made it import available
    StateError,  # made it import available
    MachineError,  # made it import available
    )

from ginsfsm.gconfig import GConfig


class ParentError(Exception):
    """ Raised when parent is already defined."""


class DestinationError(Exception):
    """ Raised when destination is not know."""


class GObjError(Exception):
    """ Raised when an object is not a GObj type, and must be!."""


class NotUniqueGObjError(Exception):
    """ Raised when an named object is not the unique gobj."""


class QueueError(Exception):
    """ Raised when there is no support for enqueue objs, i.e., use of
        GObj.post_event() method.
    """


class Event(object):
    """ Collect event properties. This is the argument received by actions.

    :param destination: destination gobj whom to send the event.
    :param event_name: event name.
    :param source: list with path of gobj sources. Firt item ``source[0]``
        is the original sender gobj. Last item ``source[-1]`` is the
        nearest sender gobj.
    :param kw: keyword arguments with associated data to event.
    """
    # For now, event_factory is private. Automatically using by send_event...
    #Use the :meth:`GObj.event_factory` factory function to create Event
    #instances.
    def __init__(self, destination, event_name, source, **kw):
        self.destination = destination
        self.event_name = event_name
        if not isinstance(source, (list, tuple)):
            source = [source]
        self.source = source
        self.kw = kw
        self.__dict__.update(**kw)


class _Subscription(object):
    """ Collect subscriber properties
    `event_name`: event name.
    `subscriber_gobj`: subcriber gobj to sending event.
    `kw`: event parameters
    """
    def __init__(self, event_name, subscriber_gobj, **kw):
        self.event_name = event_name
        self.subscriber_gobj = subscriber_gobj
        self.kw = kw
        self.__dict__.update(**kw)


# Attributes that a gaplic can update.
GOBJ_GCONFIG = {
    'gaplic': [None, None, 0, None, ''],
    'trace_dump': [bool, False, 0, None, 'Display hexa input-ouput data'],
    # trace_mach is inherited from SMachine.
    'trace_mach': [bool, False, 0, None, 'Display simple machine activity'],
    # logger is inherited from SMachine.
    'logger': [None, None, 0, None, ''],
    'create_gobj': [None, None, 0, None, ''],
    'enqueue_event': [None, None, 0, None, ''],
    'get_named_gobjs': [None, None, 0, None, ''],
    'register_unique_gobj': [None, None, 0, None, ''],
    'deregister_unique_gobj': [None, None, 0, None, ''],
    'search_unique_gobj': [None, None, 0, None, ''],
    '_increase_inside': [None, None, 0, None, ''],
    '_decrease_inside': [None, None, 0, None, ''],
    '_tab': [None, None, 0, None, ''],
}


class GObj(SMachine, GConfig):
    """ Well, yes, I'm a very simple brain. Only a machine.
    But write a good FSM, and I never fail you. Derive me, and write my FSM.

    Sample GObj::

        class MyGObj(GObj):
            def __init__(self):
                GObj.__init__(self, FSM, GCONFIG)

            def start_up(self):
                ''' Initialization zone.'''

    :param fsm: FSM :term:`simple-machine`.
    :param gconfig: GCONFIG :term:`gconfig-template`.
    """
    def __init__(self, fsm, gconfig=None):
        SMachine.__init__(self, fsm)
        self.name = ''
        """ My name.
        Set by :meth:`create_gobj`
        """
        self.parent = None
        """My parent, destination of my events... or not.
        Set by :meth:`create_gobj`
        """
        self.dl_childs = set()        # my childs... me too like be parent.
        """List of gobj childs.
        """
        self.owned_event_filter = None  # TODO debe ser una lista de filtros
        """Filter to broadcast_event function to check the owner of events.
        """
        self.global_settings = None

        self._dl_subscriptions = set()      # uauuu, how many fans!!
        self._some_subscriptions = False

        if gconfig is not None:
            if isinstance(gconfig, (list, tuple)):
                gconfig = list(gconfig)
                gconfig.append(GOBJ_GCONFIG)
            else:
                gconfig = [gconfig, GOBJ_GCONFIG]
        else:
            gconfig = [GOBJ_GCONFIG]
        GConfig.__init__(self, gconfig)

    def create_gobj(self, name, gclass, parent, **kw):
        """ Factory function to create gobj's instances.

        :param name: Name of gobj. If it's a :term:`named-gobj` then the
            :meth:`GObj.register_unique_gobj` will be called.
            :meth:`GObj.register_unique_gobj` is a empty method that
            must be override by a superior domain like :term:`gaplic`.
            If :meth:`GObj.register_unique_gobj` fails, a
            :exc:NotUniqueGObjError exception will be raised.
        :param gclass: `gclass` is the :class:`GObj` type used to create
            the new gobj. It's must be a derived class of :class:`GObj`
            otherwise a :exc:`GObjError` exception will be raised.
        :param parent: parent of the new :term:`gobj`. ``None`` if it has no
            parent. It it's not ``None``, then must be a derived class
            of :class:`GObj` otherwise a :exc:`GObjError`
            exception will be raised.
        :param kw: Attributes that are added to the new :term:`gobj`.
            All the keyword arguments used in the function
            **are added as attributes** to the created :term:`gobj`.
            You must consult the attributes supported
            by each `gclass` type.
        :rtype: new gobj instance.

        The factory funcion does:

        * Add the :term:`gobj` to their parent child list
          :attr:`GObj.dl_childs`,
        * If it's a :term:`named-gobj` call the
          :meth:`GObj.register_unique_gobj`.
        * Call :meth:`GObj.start_up`.
        * Add to the :term:`gobj` several attributes:

            * **name**: name of the created :term:`gobj`.
            * **parent**: the parent :term:`gobj` of the created :term:`gobj`.

        .. warning:: the :meth:`GObj.register`, :meth:`GObj.deregister`
            and :meth:`GObj.search` methods must be override and supplied by a
            superior instance, like :term:`gaplic`,
            otherwise :term:`named-gobj` objects cannot be used.
        """

        if gclass is None:
            raise GObjError(
                '''ERROR create_gobj(): No GObj class supplied.''')
        if not issubclass(gclass, GObj):
            raise GObjError(
                '''ERROR create_gobj(): class '%s' is NOT a GObj subclass''' % (
                    repr(gclass)))

        if parent is not None:
            if not isinstance(parent, GObj):
                raise GObjError(
                    '''ERROR create_gobj(): parent '%s' '''
                    '''is NOT a GObj subclass''' % (
                        repr(gclass)))

        gobj = gclass()
        if parent is not None:
            parent._add_child(gobj)

        if name:
            gobj.name = name
            registered = self.register_unique_gobj(gobj)
            if not registered:
                raise NotUniqueGObjError(
                    '''ERROR create_gobj(): gobj '%s' '''
                    ''' is the unique named''' % (name))

        # What must be the first? arguments or global settings??
        gobj.write_parameters(**kw)
        if self.global_settings is not None:
            # global wins.
            gobj.overwrite_parameters(0, **self.global_settings)
        gobj.start_up()
        return gobj

    def destroy_gobj(self, gobj):
        """ Destroy a gobj
        """
        gobj.deregister_unique_gobj(gobj)
        if gobj.parent is not None:
            gobj.parent._remove_child(gobj)

        while len(gobj.dl_childs):
            try:
                for child in gobj.dl_childs:
                    self.destroy_gobj(child)
            except RuntimeError:
                pass  # "Set changed size during iteration" is OK

        gobj.delete_all_subscriptions()

        del gobj

    def start_up(self):
        """ Initialization zone.

        Well, the __init__ method is used to build the FSM so I need another
        function to initialize the new gobj.
        Please, **override me**, and write here all the code you need to
        start up the machine: create your owns childs, etc.
        This function is called by :meth:`create_gobj`
        after creating the gobj instance.
        """

    def _resolv_destination(self, destination):
        """ Resolv the destination :term:`gobj`.

        :param destination: destination gobj whom to send the event.
        :rtype: Return the destination :term:`gobj` instance.

        If ``destination`` is a `string`then
        it will be search in the current :term:`gaplic` domain
        with :meth:`search_unique_gobj` method.
        If it is not found a :exc:`DestinationError` exception will be raised.
        """
        if not (isinstance(destination, string_types) or
                isinstance(destination, GObj)):
            raise DestinationError(
                '_resolv_destination() BAD TYPE destination %s in (%s:%s)' %
                (repr(destination), self.__class__.__name__, self.name))

        if isinstance(destination, string_types):
            named_gobj = self.search_unique_gobj(destination)
            if not named_gobj:
                raise DestinationError(
                    '_resolv_destination() destination %s NOT FOUND in (%s:%s)'
                    '\nCurrent named-gobjs: %s'
                    % (repr(destination),
                       self.__class__.__name__,
                       self.name,
                       ' '.join(sorted(self.get_named_gobjs())),
                       )
                    )
            destination = named_gobj
        return destination

    def _event_factory(self, destination, event, **kw):
        """ Factory to create Event instances.

        :param destination: destination gobj whom to send the event.
        :param event: an :term:`event`.
        :param kw: keyword arguments with associated data to event.
        :rtype: Return Event instance.

        ``event`` must be a `string` or :class:`Event` types, otherwise a
        :exc:`EventError` will be raised.

        If ``event`` is an :class:`Event` instance, a new :class:`Event`
        duplicated instance is returned, but it will be updated with
        the new ``destination`` and ``kw`` keyword arguments.

        .. note::
            All the keyword arguments used in the factory function
            **are added as attributes** to the created :term:`event` instance.
            You must consult the attributes supported by each machine's event.
        """
        if not (isinstance(event, string_types) or
                isinstance(event, Event)):
            raise EventError('_event_factory() BAD TYPE event %s in (%s:%s)' %
                (repr(event), self.__class__.__name__, self.name))

        if destination is not None:
            if not (isinstance(destination, string_types) or
                    isinstance(destination, GObj)):
                raise DestinationError(
                    '_event_factory() BAD TYPE destination %s in (%s:%s)' %
                    (repr(destination), self.__class__.__name__, self.name))

        if isinstance(event, Event):
            # duplicate the event
            if event.source[-1] != self:
                event.source.append(self)
            event = Event(event.destination, event.event_name,
                    event.source, **event.kw)
            if len(kw):
                event.__dict__.update(**kw)
            if destination is not None:
                event.destination = destination
        else:
            event = Event(destination, event, self, **kw)

        return event

    def send_event(self, destination, event, **kw):
        """
        Send **right now** the :term:`event` ``event`` to the
        destination gobj ``destination``, with associated data ``kw``.

        :param event: :term:`event` to send.
        :param destination: destination :term:`gobj` of the event.
        :param kw: keyword argument with data associated to event.
        :rtype: return the returned value from the executed action.

            .. note::
                All the keyword arguments **are added as attributes** to
                the sent :term:`event`.

        If ``event`` argument is a string type, then the internal
        :func:`_event_factory` function  is called, to create a :class:`Event`
        instance.

        The algorithm to calculate the destination :term:`gobj` will be:

            1. ``destination`` if the ``destination`` argument is not ``None``.
            2. ``event.destination`` if ``event`` is a Event instance and
               ``event.destination`` is not ``None``.
            3. If ``destination`` and ``event.destination`` is None, then
               a :exc:`DestinationError` exception will be raised.

        If the :term:`event-name` exists in the machine, but it's not accepted
        by the current state, then no exception is raised but the
        function **returns** :exc:`EventNotAcceptedError`.

            .. note:: The :meth:`inject_event` method doesn't
                **raise** :exc:`EventNotAcceptedError` because a :term:`machine`
                should run under any circumstances. In any way an action can
                raise exceptions.

        If ``destination`` is a :term:`named-gobj`, i.e. a string, then the gobj
        will be search with :meth:`search_gobj` method. This method must be
        supplied and override by superior instance, like :term:`gaplic`.

        ``destination`` must be a `string` or :class:`GObj` types, otherwise a
        :exc:`GObjError` will be raised.

        If ``destination`` if the recipient runs
        in another :term:`gaplic` that the sender,
        the :meth:`post_event` method will be used.

        ``event`` must be a `string` or :class:`Event` types, otherwise a
        :exc:`EventError` will be raised.

        If ``event`` is an :class:`Event` instance, a new :class:`Event`
        duplicated instance is returned, but it will be updated with
        the new ``destination`` and ``kw`` keyword arguments.

        .. note::
            All the keyword arguments used in the factory function
            **are added as attributes** to the created :term:`event` instance.
            You must consult the attributes supported by each machine's event.
        """
        destination = self._resolv_destination(destination)
        event = self._event_factory(destination, event, **kw)

        if self.gaplic:
            cur_ident = threading.current_thread().ident
            cur_name = threading.current_thread().name
            if self.gaplic.thread_ident and cur_ident != self.gaplic.thread_ident:
                logging.debug("ERROR internal: "  # pragma: no cover
                    "current thread '%s' is not the sender thread '%s'",
                    cur_name, self.gaplic.thread_name)

            dst_ident = destination.gaplic.thread_ident
            if dst_ident and cur_ident != dst_ident:
                return self.post_event(destination, event)

        ret = destination.inject_event(event)
        return ret

    def post_event(self, destination, event, **kw):
        """ Post the event in the event queue. To use with domains like
        :term:`gaplic` because it's necessary to override :meth:`enqueue_event`
        in order to supply a queue system.

        :param event: :term:`event` to send.
        :param destination: destination :term:`gobj` of the event.
        :param kw: keyword argument with data associated to event.

            .. note::
                All the keyword arguments **are added as attributes** to
                the sent :term:`event`.

        ``destination`` must be a `string` or :class:`GObj` types, otherwise a
        :exc:`GObjError` will be raised.

        ``event`` must be a `string` or :class:`Event` types, otherwise a
        :exc:`EventError` will be raised.

        If ``event`` is an :class:`Event` instance, a new :class:`Event`
        duplicated instance is returned, but it will be updated with
        the new ``destination`` and ``kw`` keyword arguments.

        .. note::
            All the keyword arguments used in the factory function
            **are added as attributes** to the created :term:`event` instance.
            You must consult the attributes supported by each machine's event.
        """
        event = self._event_factory(destination, event, **kw)
        self.enqueue_event(event)

    def enqueue_event(self, event):
        """ Enqueue a event to send by queue system.
        To be overriden by :term:`gaplic` or similar
        """
        raise QueueError(
              'enqueue_event() no support in (%s:%s)' % (
              self.__class__.__name__, self.name))

    def broadcast_event(self, event, **kw):
        """ Broadcast the ``event`` to all subscribers.

        :param event: :term:`event` to send.
        :param kw: keyword argument with data associated to event.

            .. note::
                All the keyword arguments **are added as attributes** to
                the sent :term:`event`.

        Use this function when you don't know who are your event's clients,
        when you don't know the :term:`gobj` destination of
        your :term:`output-event`'s

        If there is no subscriptors, the event is not sent.

        When an event has several subscriptors, there is a mechanism called
        :term:`event-filter` that allows to a subcriptor to own the event
        and no further spread by more subscribers.

        The filter function set by :meth:`set_owned_event_filter` method,
        is call with the returned value of an :term:`action` as argument:
        If the filter function return ``True``, the event is owned, and the
        :func:`ginsfsm.gobj.GObj.broadcast_event` function doesn't continue
        sending the event to other subscribers.

        .. note:: If :func:`ginsfsm.gobj.GObj.broadcast_event` function
           uses :func:`ginsfsm.gobj.GObj.post_event`,
           the :term:`event-filter` cannot be applied.
        """
        if self._some_subscriptions:
            event = self._event_factory(None, event, **kw)
            subscriptions = self._dl_subscriptions.copy()
            sended_gobj = set()  # don't repeat events
            for sub in subscriptions:
                if sub.subscriber_gobj in sended_gobj:
                    continue
                if None in sub.event_name or event.event_name in sub.event_name:
                    if hasattr(sub, 'use_post_event'):
                        ret = self.post_event(sub.subscriber_gobj, event)
                    else:
                        ret = self.send_event(sub.subscriber_gobj, event)
                    sended_gobj.add(sub.subscriber_gobj)
                    if self.owned_event_filter:
                        ret = self.owned_event_filter(ret)
                        if ret is True:
                            return True  # propietary event, retorno otra cosa?

    def subscribe_event(self, event_name, subscriber_gobj, **kw):
        """ Subscribe to event.

        :param event_name: string event name or tuple/list of string
            event names.  If ``event_name`` is ``None`` then it subscribe
            to all events. If it's not ``None`` then it must be a valid event
            name from the :term:`output-event` list,
            otherwise a :exc:`EventError` will be raised.
        :param subscriber_gobj: subscriber obj that wants receive the event.
            If ``subscriber`` is ``None`` then the subscriber is the parent.
            ``subscriber_gobj`` must be `None` or a `string` or a
            :class:`GObj` types, otherwise a :exc:`GObjError` will be raised.
        :param kw: keyword argument with data associated to subscription.

        **kw** values:
            * `use_post_event`:
              You must set it to `True` in order to broadcast the events
              using `post-event` instead of `send-event`.

        """
        if subscriber_gobj is None:
            subscriber_gobj = self.parent

        if subscriber_gobj is not None:
            if not (isinstance(subscriber_gobj, string_types) or
                    isinstance(subscriber_gobj, GObj)):
                raise GObjError(
                    'subcribe_event(): BAD TYPE subscriber_gobj %s in (%s:%s)' %
                    (repr(subscriber_gobj), self.__class__.__name__, self.name))

            if isinstance(subscriber_gobj, string_types):
                named_gobj = self.search_unique_gobj(subscriber_gobj)
                if not named_gobj:
                    raise GObjError(
                        'subscribe_event() subscriber NOT FOUND in (%s:%s)'
                        '\nCurrent named-gobjs: %s'
                        % (self.__class__.__name__,
                           self.name,
                           ' '.join(sorted(self.get_named_gobjs())),
                           )
                        )
                subscriber_gobj = named_gobj

        output_events = self.get_output_event_list()

        if not isinstance(event_name, (list, tuple)):
            event_name = (event_name,)

        for name in event_name:
            if name is None:
                continue
            if not isinstance(name, string_types):
                raise EventError(
                    'subscribe_event(): event %s is not string in (%s:%s)'
                    % (repr(name), self.__class__.__name__, self.name))

            if name not in output_events:
                raise EventError(
                    'subscribe_event(): output-event %s not defined in'
                    ' (%s:%s)' % (
                        repr(event_name),
                        self.__class__.__name__,
                        self.name))

        existing_subs = self._find_subscription(event_name, subscriber_gobj)
        if existing_subs:
            # avoid duplication subscriptions
            self.delete_subscription(event_name, subscriber_gobj)
        subscription = _Subscription(event_name, subscriber_gobj, **kw)
        self._dl_subscriptions.add(subscription)
        self._some_subscriptions = True

    def _find_subscription(self, event_name, subscriber_gobj):
        """ Find a subscription by event_name and subscriber gobj.
        Internal use to avoid duplicates subscriptions.
        """
        if not isinstance(event_name, (list, tuple)):
            event_name = (event_name,)
        for sub in self._dl_subscriptions:
            if sub.event_name == event_name and \
                sub.subscriber_gobj == subscriber_gobj:
                    return sub

    def delete_subscription(self, event_name, subscriber_gobj):
        """ Remove `subscription`.

        :param event_name: string event name or tuple/list of string
            event names.
        :param subscriber_gobj: subscriber gobj.
        """
        existing_subs = self._find_subscription(event_name, subscriber_gobj)
        if existing_subs:
            self._dl_subscriptions.remove(existing_subs)
            if len(self._dl_subscriptions) == 0:
                self._some_subscriptions = False
            return True
        return False

    def delete_all_subscriptions(self):
        """ Remove all subscriptions.
        """
        while len(self._dl_subscriptions):
            try:
                for subs in self._dl_subscriptions:
                    self._dl_subscriptions.remove(subs)
            except RuntimeError:
                pass  # "Set changed size during iteration" is OK

    def set_owned_event_filter(self, filter):
        """ Set a filter function to be used by :meth:`broadcast_event` function
        to check the owner of events.
        """
        self.owned_event_filter = filter

    def get_named_gobjs(self):
        """ Return the list of :term:`named-gobj`'s.
        To be overriden by :term:`gaplic` or similar.
        """
        return []

    def register_unique_gobj(self, gobj):
        """ Register a :term:`named-gobj`.
        To be overriden by :term:`gaplic` or similar.
        """
        return True

    def deregister_unique_gobj(self, gobj):
        """ Deregister a :term:`named-gobj`.
        To be overriden by :term:`gaplic` or similar.
        """
        return True

    def search_unique_gobj(self, gobj_name):
        """ Search a :term:`named-gobj`.
        To be overriden by :term:`gaplic` or similar.
        """
        return None

    def _add_child(self, gobj):
        """ Add a child ``gobj``.

        :param gobj: :term:`gobj` child to add.

        Raise :exc:`ParentError` is ``gobj`` already has a parent.
        This function is called by :meth:`create_gobj`
        after creating the gobj instance.
        """
        if gobj.parent:
            raise ParentError(
                '_add_child(): gobj already has parent in (%s:%s)' %
                (self.__class__.__name__, self.name))
        self.dl_childs.add(gobj)
        gobj.parent = self

    def _remove_child(self, gobj):
        """ Remove the child ``gobj``.

        :param gobj: :term:`gobj` child to remove.

        This function is called by :meth:`destroy_gobj`.
        """
        if gobj in self.dl_childs:
            self.dl_childs.remove(gobj)
            gobj.parent = None

    def __getitem__(self, name):
        """ Enable gobjs tree to work with Pyramid trasversal URL dispatch.
        """
        if name is None:
            raise KeyError('No such child named None')
        for gobj in self.dl_childs:
            if gobj.name == name:
                return gobj
        raise KeyError('No such child named %s' % name)

    def __iter__(self):
        """ Iterates over child elements.
        """
        return self.dl_childs.__iter__()

    def overwrite_parameters(self, level, **settings):
        """ The parameters in settings must be defined in the gobj.

        :param level: level trace of childs.
            indicates the depth of the childs as far to change.

            * `0` only this gobj.
            * `-1` all tree of childs.

        :param settings: parameters and their values.

        The settings are filtered by the named-gobj or gclass name of this gobj.
        The parameter name in settings, must be a dot-named,
        with the first item being the named-gobj o gclass name.
        """
        parameters = self.filter_parameters(**settings)
        self.write_parameters(**parameters)
        if level:
            for child in self.dl_childs:
                child.overwrite_parameters(level - 1, **settings)

    def overwrite_few_parameters(self, parameter_list, **settings):
        """ The parameters in settings must be defined in the gobj.

        :param parameters: write only the parameters in ``parameter_list``.

        :param settings: parameters and their values.

        The settings are filtered by the named-gobj or gclass name of this gobj.
        The parameter name in settings, must be a dot-named,
        with the first item being the named-gobj o gclass name.
        """
        parameters = self.filter_parameters(**settings)
        self.write_few_parameters(parameter_list, **parameters)



# -*- coding: utf-8 -*-
""" Porting of GSMG.C
"""
import logging

from ginsfsm.compat import string_types
from .utils import string_to_bytearray

from .dl_list import (
    ListItem,
    DoubleList,
    dl_add,
    dl_first,
    dl_last,
    dl_next,
    dl_delete,
    )


class SegmError(Exception):
    """ Raised when Segm have problems."""


class GMsgError(Exception):
    """ Raised when GMsg have problems."""


class Segm(ListItem):
    def __init__(self, gmsg):
        super(Segm, self).__init__()
        #
        # El número actual de bytes sin procesar es (tail - head).
        # El máximo número de bytes que puede haber es 'segm_size'.
        # 'tail' no puede ser mayor que 'segm_size'
        #
        self.head = 0  # primer byte sin procesar. Usado para LECTURA
        self.tail = 0  # próximo byte a añadir. Usado para ESCRITURA (añadir)
        self.fixp = 0  # fixed pointer. Usado en ESCRITURA para sobreescribir

        #
        #  Calcula tamaño segmento
        #
        if gmsg.cur_size >= gmsg.max_size:
            msg = "segm_create(): MAXIMUM size reached (%d over %d)" % (
                gmsg.cur_size, gmsg.max_size)
            logging.warning(msg)
            raise SegmError(msg)  # Alcanzado máximo tamaño

        data_size = gmsg.max_size - gmsg.cur_size
        if data_size > gmsg.segm_size:
            data_size = gmsg.segm_size
        gmsg.cur_size += data_size

        self.segm_size = data_size
        # Datos allocados dinamicamente (segm_size)
        self.data = bytearray(data_size)


class GMsg(ListItem):
    def __init__(self, segm_size, max_size=0):
        self.segm_size = segm_size  # tamaño segmentos datos
        # tamaño máximo total, si es 0, igual a segm_size (like PACKET!!!)
        self.max_size = max_size if max_size > 0 else segm_size
        self.cur_size = 0  # tamaño total actual
        self.dl_segm = DoubleList()  # lista de segmentos de datos
        self.fix_segm = None  # SEGM segmento marked para escritura (OVERWRITE)
        self.rd_segm = None  # SEGM segmento de lectura
        self.sublen = 0  # submensaje. procesamiento parcial de un msg

        #
        #   Creo segmento inicial
        #
        try:
            segm = Segm(self)
        except SegmError:
            raise GMsgError("ERROR creating segm")
        dl_add(self.dl_segm, segm)
        self.reset_rd()
        self.reset_wr()

    def segm_create(self):
        """ Crea un segmento de datos de tamaño 'data_size'
            Retorna None if error
        """
        try:
            segm = Segm(self)
        except SegmError:
            return None

        return segm

    def reset_rd(self):
        """ Reset current READING pointer
        """
        #
        #  Set current segment to first segment
        #
        self.rd_segm = dl_first(self.dl_segm)

        #
        #  Set ALL current segment pointer to 0
        #
        segm = dl_first(self.dl_segm)
        while segm:
            segm.head = 0
            segm = dl_next(segm)

    def reset_wr(self):
        """ Reset current WRITING pointer
        """
        #
        #  Set current segment to first segment
        #
        self.fix_segm = dl_first(self.dl_segm)

        #
        #  Set ALL current segment pointer to 0
        #
        segm = dl_first(self.dl_segm)
        while segm:
            segm.tail = 0
            segm.fixp = 0
            segm.data = bytearray(segm.segm_size)  # memset()
            segm = dl_next(segm)

    def getchar(self):
        """ LECTURA o procesamient del mensaje.
            Saca un byte del message retornalo
            Retorna None si no hay más bytes
        """
        if not self.rd_segm:
            #
            #   No hay mas datos
            #
            return None

        while self.rd_segm:
            if self.rd_segm.head < self.rd_segm.tail:
                #
                #  Retorna el byte y eliminalo del segm
                #
                c = self.rd_segm.data[self.rd_segm.head]
                self.rd_segm.head += 1
                return c

            #
            #   No hay mas datos en el segm
            #   Intentalo en el siguiente
            #
            self.rd_segm = dl_next(self.rd_segm)

        return None

    def check_char(self):
        """ LECTURA o procesamient del mensaje.
            Consulta un byte del message y dejalo en 'c' (NO LO SACA DEL MSG)
            Retorna None si no hay mas bytes, y el caracter si lo hay.
        """
        if not self.rd_segm:
            return None

        while self.rd_segm:
            if self.rd_segm.head < self.rd_segm.tail:
                #
                #  Retorna el byte
                #
                c = self.rd_segm.data[self.rd_segm.head]
                return c

            #
            #   No hay mas datos en el segm
            #   Intentalo en el siguiente
            #
            self.rd_segm = dl_next(self.rd_segm)

        return None

    def cur_seg_rd(self):
        """  LECTURA : Devuelve segm lectura actual
        """
        if not self.rd_segm:
            return None
        return self.rd_segm.data[self.rd_segm.head:]

    def cur_seglen_rd(self):
        """  LECTURA : Devuelve ln segm lectura actual
        """
        if not self.rd_segm:
            return None
        return self.rd_segm.tail - self.rd_segm.head

    def next_seg_rd(self):
        """ LECTURA : Pon como actual el siguiente segm de lectura
            Retorna false si no hay mas.
        """
        if not self.rd_segm:
            return None
        self.rd_segm = dl_next(self.rd_segm)
        if self.rd_segm:
            return True
        else:
            return False

    def getdata(self, ln):
        """ LECTURA : Saca 'ln' bytes del self, moviendo los datos a 'data'
            desde la posición actual de lectura.
            NOO Retorna nº de bytes sacados
            Retorna los datos
        """
        if not self.rd_segm:
            return None

        data = bytearray()
        while self.rd_segm and ln > 0:
            if self.rd_segm.head < self.rd_segm.tail:
                seg_len = self.rd_segm.tail - self.rd_segm.head
                if seg_len >= ln:
                    #
                    #  Con los bytes de este segm hay suficiente
                    #
                    data = self.rd_segm.data[
                        self.rd_segm.head:self.rd_segm.head + ln]
                    self.rd_segm.head += ln
                    return data
                else:
                    #
                    #  Falta bytes con este segm
                    #
                    data += self.rd_segm.data[
                        self.rd_segm.head:self.rd_segm.head + seg_len]

                    self.rd_segm.head += seg_len
                    ln -= seg_len

            #
            #   No hay mas datos en el segm
            #   Intentalo en el siguiente
            #
            self.rd_segm = dl_next(self.rd_segm)

        #
        #   No hay mas datos
        #
        return data

    def subsetdata(self, ln):
        """ LECTURA : Fija 'ln' bytes para el submensaje
        """
        if ln < self.bytesleft():
            self.sublen = ln
            return True
        logging.warning("Not enough data for subsetdata: (%d over %d)" % (
                        ln, self.bytesleft()))
        self.sublen = 0
        return False

    def subgetdata(self, ln):
        """ LECTURA : Saca 'ln' bytes del SUB-MSG, moviendo los datos a 'data'
            desde la posición actual de lectura.
            Si 'data' es NULL simplemente se sacan los bytes,
            sin copiarlos a nada.
            Retorna TRUE si n§ de bytes sacados es 'ln'
        """
        if ln > self.sublen:
            #
            #   No hay suficientes datos en el sublen
            #
            return None
        self.sublen -= ln
        return self.getdata(ln)

    def remove_subdata(self):
        # LECTURA : Saca los bytes del SUB-MSG, eliminandolos
        return self.subgetdata(self, self.sublen)

    def putchar(self, c):
        """ ESCRITURA del mensaje.
            Añade un byte al final del mensaje
            Retorna numero bytes escritos.
        """
        return self.putdata(self, c, 1)

    def putdata(self, data, ln=0):
        """ ESCRITURA del mensaje.
            Añade 'ln' bytes al final del mensaje
            Retorna numero bytes escritos.
        """
        data = string_to_bytearray(data)
        if ln == 0:
            ln = len(data)
        seg_len = 0
        writted = 0

        while ln > 0:
            segm = dl_last(self.dl_segm)
            if not segm or segm.tail >= segm.segm_size:
                #
                #   Crea un nuevo segm
                #
                segm = self.segm_create()
                if not segm:
                    return writted
                dl_add(self.dl_segm, segm)

            seg_len = segm.segm_size - segm.tail
            if seg_len >= ln:
                #
                #   Hay espacio en este segm
                #
                segm.data[segm.tail:ln] = data[:ln]
                segm.tail += ln
                writted += ln
                return writted
            else:
                #
                #   NO suficiente espacio
                #   Guarda lo que quepa en el segmento actual
                #
                segm.data[segm.tail:seg_len] = data[:seg_len]
                segm.tail += seg_len
                ln -= seg_len
                data = data[seg_len:]
                writted += seg_len
                #
                #   Crea un nuevo segm
                #
                segm = self.segm_create()
                if not segm:
                    return writted
                dl_add(self.dl_segm, segm)
        return writted

    def unputdata(self, ln):
        """ DESESCRITURA del mensaje.
            Quita 'ln' bytes del final del mensaje
        """
        segm = dl_last(self.dl_segm)
        if not segm or segm.tail < ln:
            logging.error("gmsg_unputdata(): ERROR NO HAY SUFICIENTE LEN")
            return False
        segm.tail -= ln
        return True

    def new_segm(self):
        """ ESCRITURA del mensaje.
            Crea un nuevo segm para a€adir datos
            Retorna FALSE si error
        """
        segm = self.segm_create()
        if not segm:
            logging.debug("gmsg_new_segm(): ERROR CREANDO SEGMENTO")
            return False
        dl_add(self.dl_segm, segm)
        return True

    def cur_seg_wr(self):
        """ ESCRITURA del mensaje.
            Retorna actual pointer segm de escritura
        """
        segm = dl_last(self.dl_segm)
        if segm:
            return segm.data
        else:
            return None

    def cur_seglen_wr(self):
        """ ESCRITURA del mensaje.
            Retorna len of segm de escritura actual
        """
        segm = dl_last(self.dl_segm)
        if segm:
            return segm.tail
        else:
            return 0

    def insert_char(self, c):
        """ ESCRITURA del mensaje.
            Inserta al principio del mensaje
            !!! OJO QUE INSERTA EN head
            Retorna FALSE si error
        """
        return self.insert_data(self, c, 1)

    def insert_data(self, data, ln=0):
        """ ESCRITURA del mensaje.
            Inserta 'ln' bytes al principio del mensaje
            !!!OJO QUE SE INSERTA EN HEAD
            !! NO SE PUEDEN INSERTAR MAS BYTES QUE EL TAMAÑO DEL SEGMENTO!!!
            Retorna FALSE si error
        """
        data = string_to_bytearray(data)
        if ln == 0:
            ln = len(data)
        if ln > self.segm_size:
            logging.error("gmsg_insert_data(): DATA TOO LARGE %d", ln)
            return False

        segm = dl_first(self.dl_segm)
        if not segm:
            logging.error("ERROR gmsg_insert_data(): No first segm")
            return False

        seg_len = segm.segm_size - segm.tail
        if seg_len >= ln:
            #
            #   Hay espacio en este segm
            #
            #   Mueve todo el segm ln posiciones
            segm.data[ln:segm.tail + ln] = segm.data[0:segm.tail]
            segm.data[0:ln] = data[0:ln]
            segm.tail += ln
            return True
        logging.warning("Cannot insert data: full data.")
        return False  # NO debe pasar nunca por aqui

    def mark_pos_write(self):
        """ Marca actual posición de escritura de datos
        """
        #
        #  Get last segment
        #
        self.fix_segm = dl_last(self.dl_segm)

        #
        #  Get tail pointer
        #
        self.fix_segm.fixp = self.fix_segm.tail

    def overwrite_data(self, data, ln=0):
        """ ESCRITURA del mensaje.
            Sobreescribe en la marked (fixed) position
            Retorna FALSE si error
        """
        if not self.fix_segm:
            #
            #  No hay fixed segm
            #
            logging.error("ERROR gmsg_overwrite_data() NO FIXED SEGM")
            return 0

        data = string_to_bytearray(data)
        if ln == 0:
            ln = len(data)

        self.fix_segm = self.fix_segm
        while self.fix_segm and ln > 0:
            if self.fix_segm.fixp < self.fix_segm.tail:
                seg_len = self.fix_segm.tail - self.fix_segm.fixp
                if seg_len >= ln:
                    #
                    #  Hay espacio en este segm
                    #
                    pos = self.fix_segm.fixp
                    self.fix_segm.data[pos:pos + ln] = data[:ln]
                    self.fix_segm.fixp += ln
                    return True
                else:
                    #
                    #   NO suficiente espacio
                    #
                    pos = self.fix_segm.fixp
                    self.fix_segm.data[pos:pos + seg_len] = data[:seg_len]
                    self.fix_segm.fixp += seg_len
                    ln -= seg_len
                    data = data[seg_len:]

            #
            #   No espacio en el segm
            #   Intentalo en el siguiente
            #
            self.fix_segm = dl_next(self.fix_segm)

        return False  # can't override all

    def marked_bytes(self):
        """ ESCRITURA: numero total de bytes escritos
            desde la marca hasta el final
        """
        sum_len = 0

        segm = self.fix_segm
        if not segm:
            return 0
        sum_len = segm.tail - segm.fixp
        segm = dl_next(segm)

        while segm:
            seg_len = segm.tail - segm.fixp
            if seg_len > 0:
                sum_len += seg_len
            segm = dl_next(segm)
        return sum_len

    def bytesleft(self):
        """LECTURA: nº bytes pendientes de procesar
        """
        sum_len = 0

        if not self.rd_segm:
            #
            #   No hay mas datos para leer
            #
            return 0

        segm = self.rd_segm
        while segm:
            seg_len = segm.tail - segm.head
            if seg_len > 0:
                sum_len += seg_len
            segm = dl_next(segm)

        return sum_len

    def totalbytes(self):
        """ ESCRITURA: numero total de bytes escritos
        """
        sum_len = 0

        segm = dl_first(self.dl_segm)
        while segm:
            seg_len = segm.tail
            if seg_len > 0:
                sum_len += seg_len
            segm = dl_next(segm)
        return sum_len


def gmsg_remove(gmsg):
    """ Elimina paquete
    """

    #
    #  Libera los segmentos
    #
    while 1:
        segm = dl_first(gmsg.dl_segm)
        if segm is None:
            break
        dl_delete(gmsg.dl_segm, segm)

    del gmsg



import threading


class ThreadLocalManager(threading.local):
    def __init__(self, default=None):
        # http://code.google.com/p/google-app-engine-django/issues/detail?id=119
        # we *must* use a keyword argument for ``default`` here instead
        # of a positional argument to work around a bug in the
        # implementation of _threading_local.local in Python, which is
        # used by GAE instead of _thread.local
        self.stack = []
        self.default = default

    def push(self, info):
        self.stack.append(info)

    set = push # b/c

    def pop(self):
        if self.stack:
            return self.stack.pop()

    def get(self):
        try:
            return self.stack[-1]
        except IndexError:
            return self.default()

    def clear(self):
        self.stack[:] = []


manager = ThreadLocalManager(default=defaults)


def get_thread_data(key):
    return manager.get()[key]




"""
GObj :class:`OnServer`
======================

Stress with many connections and many data.

To run with :mod:`ginsfsm.examples.stress_client_echo`

The server echo the data.

It uses :class:`ginsfsm.c_srv_sock.GServerSock`.

.. autoclass:: OnServer
    :members:

"""

import logging
logging.basicConfig(level=logging.DEBUG)

from ginsfsm.gobj import GObj
from ginsfsm.gaplic import GAplic
from ginsfsm.c_timer import GTimer
from ginsfsm.c_srv_sock import GServerSock

#===============================================================
#                   Server
#===============================================================
n_connected_clisrv = 0


def ac_clisrv_timeout(self, event):
    self.set_timeout(10)
    print "Server's clients: %d, connected %d" % (len(self.dl_childs), n_connected_clisrv)


def ac_clisrv_connected(self, event):
    global n_connected_clisrv
    n_connected_clisrv += 1


def ac_clisrv_disconnected(self, event):
    global n_connected_clisrv
    n_connected_clisrv -= 1
    self.destroy_gobj(event.source[-1])


def ac_clisrv_rx_data(self, event):
    #print 'recibo:', event.data
    # Do echo
    self.send_event(event.source[-1], 'EV_TX_DATA', data=event.data)


SERVER_FSM = {
    'event_list': (
        'EV_SET_TIMER: bottom output',
        'EV_TIMEOUT: bottom input',
        'EV_CONNECTED: bottom input',
        'EV_DISCONNECTED: bottom input',
        'EV_RX_DATA:bottom input',
    ),
    'state_list': ('ST_IDLE',),
    'machine': {
        'ST_IDLE':
        (
            ('EV_TIMEOUT',          ac_clisrv_timeout,         'ST_IDLE'),
            ('EV_CONNECTED',        ac_clisrv_connected,       'ST_IDLE'),
            ('EV_DISCONNECTED',     ac_clisrv_disconnected,    'ST_IDLE'),
            ('EV_RX_DATA',          ac_clisrv_rx_data,         'ST_IDLE'),
        ),
    }
}


SERVER_GCONFIG = {  # type, default_value, flag, validate_function, desc
    'verbose': [int, 0, 0, None, "Increase output verbosity. Values [0,1,2]"],
}


class OnServer(GObj):
    """  Server GObj.

    .. ginsfsm::
       :fsm: SERVER_FSM
       :gconfig: SERVER_GCONFIG

    *Input-Events:*

        * :attr:`'EV_TIMEOUT'`: Timer over.
            Start the machine.

        * :attr:`'EV_CONNECTED'`: New client.

        * :attr:`'EV_DISCONNECTED'`: Client disconnected.

        * :attr:`'EV_RX_DATA'`: Receiving data.

    *Output-Events:*

        * :attr:`'EV_START_TIMER'`: Start timer.

    """
    def __init__(self):
        GObj.__init__(self, SERVER_FSM, SERVER_GCONFIG)

    def start_up(self):
        self.timer = self.create_gobj(
            None,
            GTimer,
            self)
        self.set_timeout(5)

        self.server = self.create_gobj(
            None,
            GServerSock,
            self,
            host='0.0.0.0',
            port=8000,
            transmit_ready_event_name=None,
        )

        if self.verbose > 1:
            settings = {
                'GObj.trace_mach': True,
                'GObj.logger': logging,
            }
            self.overwrite_parameters(-1, **settings)

    def set_timeout(self, seconds):
        self.send_event(self.timer, 'EV_SET_TIMER', seconds=seconds)

    def clear_timeout(self):
        self.send_event(self.timer, 'EV_SET_TIMER', seconds=-1)


#===============================================================
#                   Main
#===============================================================
if __name__ == "__main__":
    ga_srv = GAplic('Server')
    ga_srv.create_gobj(
        'server',
        OnServer,
        None,
    )

    try:
        ga_srv.mt_process()
    except (KeyboardInterrupt, SystemExit):
        print('Program stopped')



# -*- encoding: utf-8 -*-
""" Convert a smachine into a layout of rectangles, lines, etc.

Sizes:
       Web       Desktop
       ------    -------
0.8em  ^= 13px
0.813em = 13px
0.875em = 14px
0.938em = 15px = 11pt
1em     = 16px = 12pt
1.063em = 17px = 13pt used by me in desktop
1.5em   = 24px

Lower letter no please, I'm too old.

Do I write the smachines with normal or reduced letter?
Right now, I think it's better work with normal letter. Again, I'm too old.
And there will always be time to change (or not?).

I choose for work with normal web apps:
    - 1em for normal letter,
    - 0.8em for a little reduced letter.

For blogs (more readability):
    - 1.063em for normal letter.

I choose for work with desktop apps:
    - 13pt for normal letter (1.063em, as blogs).

"""

from svgwrite.container import Group
import svgwrite.shapes
import svgwrite.text
import svgwrite.path
import svgwrite.animate

from ginsfsm.compat import iteritems_
from .utils import (
    get_textsize,
    Point, Size,
    colors,
    right_arrow_path,
    top_arrow_path,
    left_arrow_path,
    bottom_arrow_path,
    )
from .metrics import Metrics


class State(Group):
    """ Draw a state.

                             STATE rect

                        ----------------------
                       |  TITLE rect          | . . Event ONLY-OUT points
                       |----------------------|
    ACTION-IN points . |  ACTION rect         | . ACTION-OUT points
                     . |  ACTION rect         | .
                       |   .....              |
                     . |  ACTION rect         | .
                        ----------------------

    Everything needed to graphically represent a state.
    Requires only:
        - the names, the only variable data,
          also needed to calculate the width of the box;
          the maximum name length will be the width
          of the box, plus strokes, margins, etc (fixed data: metrics).
    """
    def __init__(self, layer, statedesc, stateid):
        """ statedesc and stateid are the state names/ids description.
        """
        super(State, self).__init__()
        self.layer = layer
        self.statedesc = statedesc
        self.stateid = stateid
        self.name = statedesc.name
        self.testing = False
        self.maximum_text_width = 0
        self.maximum_text_height = 0
        self.origin = Point(0, 0)
        self.state_rect = None
        self.blur_rect = None
        self.input_points = {}
        self.output_points = {}
        self.build_state_rect()

    def translate(self, x, y):
        self.origin = Point(x, y)
        super(State, self).translate(x, y)

    def resize(self, cx=None, cy=None):
        if cx is not None:
            self.state_rect.attribs['width'] = cx
            self.blur_rect.attribs['width'] = cx
            self.sep_line.commands[0][1][0] = cx
            for name, values  in iteritems_(self.output_points):
                output_point, connector, next_state = values
                output_point.x = cx - 2  # rectangle has a width of fixed 4.
                connector.attribs['x'] = cx - 2

        if cy is not None:
            self.state_rect.attribs['height'] = cy
            self.blur_rect.attribs['height'] = cy

    @property
    def size(self):
        return Size(self.state_rect.attribs['width'],
                    self.state_rect.attribs['height'])

    def build_state_rect(self):
        y = 0
        x_margin = self.layer.metrics.x_margin
        y_margin = self.layer.metrics.y_margin
        internal_size = self.get_maximum_text_element_size()
        external_size = internal_size.extend(x_margin * 2, y_margin)

        if 1:  # transparent_blur:
            filtr = "filter:url(#filter_blur);opacity:0.7;fill-opacity:1"
        else:
            filtr = "filter:url(#filter_blur)"

        self.blur_rect = svgwrite.shapes.Rect(
            Point(0, 0),
            Size(external_size.width, 0),
            rx=10,
            ry=10,
            fill='black',
            stroke='black',
            stroke_width=1,
            style=filtr,
        )
        self.add(self.blur_rect)

        self.state_rect = svgwrite.shapes.Rect(
            Point(0, 0),
            Size(external_size.width, 0),
            rx=10,
            ry=10,
            stroke='black',
            stroke_width=1,
            fill='#faf1e8')
        self.add(self.state_rect)

        if self.testing:
            title_rect = svgwrite.shapes.Rect(
                Point(x_margin, y_margin),
                internal_size,
                stroke='black',
                stroke_width=0.1,
                fill='white')
            self.add(title_rect)

        title_text = svgwrite.text.Text(self.statedesc.name,
            insert=Point(x_margin, y_margin + self.layer.metrics.ascent))
        self.add(title_text)

        y += external_size.height
        self.state_rect.attribs['height'] += external_size.height

        # separator line
        d = [('m', 0, y + y_margin / 2), [external_size.width, 0]]
        self.sep_line = svgwrite.path.Path(d=d, stroke='black', stroke_width=1)
        self.add(self.sep_line)

        for id in list(range(len(self.statedesc.ev_names))):
            ac_name = self.statedesc.ac_names[id]
            ev_name = self.statedesc.ev_names[id]
            next_state = self.statedesc.next_st_names[id]
            if self.testing:
                action_rect = svgwrite.shapes.Rect(
                    Point(x_margin, y + y_margin),
                    internal_size,
                    stroke='black',
                    stroke_width='0.2',
                    fill='white')
                self.add(action_rect)
            action_text = svgwrite.text.Text(ac_name,
                insert=Point(x_margin,
                             y + y_margin + self.layer.metrics.ascent))
            self.add(action_text)

            #
            #   Input point of the action (event)
            #
            input_point = Point(0, y + internal_size.height)
            self.input_points[ev_name] = input_point

            # draw a --. arrow with circle end.
            self.add(svgwrite.shapes.Line(
                input_point,
                input_point.shift(x_margin / 2),
                fill='black',
                stroke='black',
                stroke_width=1)
            )
            self.add(svgwrite.shapes.Circle(
                input_point.shift(x_margin / 2),
                r=3,
                fill='black',
                stroke='black',
                stroke_width=1)
            )

            #
            #   Output point of the action (next-state)
            #
            output_point = Point(external_size.width, y + internal_size.height)

            # draw a small square
            connector = self.add(svgwrite.shapes.Rect(
                output_point.shift(x=-2),
                (4, 4),
                fill='black',
                stroke='black',
                stroke_width=1)
            )
            # We use the ev_name because
            # the action_name and next_state can be None!.
            self.output_points[ev_name] = (output_point, connector, next_state)

            #
            #   Increment y-pointer and global state height
            #
            y += external_size.height
            self.state_rect.attribs['height'] += external_size.height

        self.state_rect.attribs['height'] += y_margin
        self.blur_rect.attribs['height'] = self.state_rect.attribs['height']

    def get_maximum_text_element_size(self):
        """ Calculate the greater element svgwrite.text area,
            including title and actions boxes
        """
        #print "%d-%s" % (stateid.id, statedesc.name)
        (width, height), ascent, descent = get_textsize(
                self.statedesc.name,
                self.layer.font_name,
                self.layer.font_size)
        self.maximum_text_width = max(self.maximum_text_width, width)
        self.maximum_text_height = max(self.maximum_text_height, height)

        for id in list(range(len(self.statedesc.ev_names))):
            # print "    ev: %d-%s -> ac: %s -> st: %d-%s" % (
            #    stateid.ev_ids[id],
            #    statedesc.ev_names[id],
            #    statedesc.ac_names[id],
            #    stateid.next_st_ids[id],
            #    statedesc.next_st_names[id],
            #)
            name = self.statedesc.ac_names[id]
            if name is None:
                continue
            (width, height), ascent, descent = get_textsize(
                name,
                self.layer.font_name,
                self.layer.font_size)
            self.maximum_text_width = max(self.maximum_text_width, width)
            self.maximum_text_height = max(self.maximum_text_height, height)
        return Size(self.maximum_text_width, self.maximum_text_height)


class StateDesc(object):
    """
    StateDesc: describe a state, his name, and three lists with the names of:
        - event name
        - action name
        - next state name
    The same index targets the same level or row of the three lists,
    in order to full the event/action/next-state trio.
    """
    def __init__(self, name, ev_names, ac_names, next_st_names):
        self.name = name
        self.ev_names = ev_names
        self.ac_names = ac_names
        self.next_st_names = next_st_names


class StateId(object):
    """
    StateId: describe the same as StateDesc, but with indexes instead of names.
    The indexes of states and events, could be better certainly
    the 'id'-entifiers, because they are uniques.
    """
    def __init__(self, id, ev_ids, next_st_ids):
        self.id = id
        self.ev_ids = ev_ids
        self.next_st_ids = next_st_ids


class States(Group):
    def __init__(self, layer, smachine):
        super(States, self).__init__()
        self.layer = layer
        self.states = []
        self.origin = Point(0, 0)
        self.smachine = smachine
        self.cx, self.cy = self.build_states()

    def build_states(self):
        """ Build the group of states.
            Return the size of group.
        """
        y = 0
        for idx, state_name in enumerate(self.smachine._state_list):
            if idx > 0:
                y += self.layer.metrics.states_y_margin

            st_id = self.smachine._state_index[state_name]
            state = self.smachine._states[st_id]
            event_name_list = []
            action_name_list = []
            next_state_name_list = []
            for ev_id, ev_ac in enumerate(state):
                if not isinstance(ev_ac, list):
                    continue
                action = ev_ac[0]
                if action is None:
                    action_name = ''
                else:
                    action_name = action.__name__
                next_state_id = ev_ac[1]
                if next_state_id is None:
                    next_state_name = ''
                else:
                    next_state_name = \
                        self.smachine._state_list[next_state_id - 1]

                event_name_list.append(self.smachine._event_list[ev_id - 1])
                action_name_list.append(action_name)
                next_state_name_list.append(next_state_name)

            statedesc = StateDesc(
                state_name,
                event_name_list,
                action_name_list,
                next_state_name_list)

            stateid = StateId(
                self.smachine._state_index[state_name],
                [self.smachine._event_index[event_name] \
                    for event_name in event_name_list],
                [self.smachine._state_index[state_name] \
                    for state_name in next_state_name_list]
            )

            state = State(self.layer, statedesc, stateid)
            state.translate(0, y)

            self.add(state)
            self.states.append(state)

            y += state.size.height

        # set all boxes and sep_line with the same width
        max_cx = 0
        for state in self.states:
            cx, cy = state.size
            max_cx = max(max_cx, cx)
        for state in self.states:
            state.resize(cx=max_cx)

        return (max_cx, y)

    def translate(self, x, y):
        self.origin = Point(x, y)
        super(States, self).translate(x, y)

    def get_input_points(self, event_name):
        points = []
        for state in self.states:
            point = state.input_points.get(event_name, None)
            if point:
                point = point.shift(state.origin.x - 1, state.origin.y)
                point = point.shift(self.origin.x - 1, self.origin.y)
                points.append(point)
        return points

    def get_output_points(self, event_name, states_filter=None):
        points = []
        for state in self.states:
            if states_filter and not state.name in states_filter:
                continue
            truplo = state.output_points.get(event_name, None)
            if truplo:
                # this output event is too an input event
                point, connector, next_state = truplo
                point = Point(connector.attribs['x'], connector.attribs['y'])
                point = point.shift(state.origin.x, state.origin.y)
                point = point.shift(self.origin.x, self.origin.y)
                point = point.shift(4, 2)
                points.append(point)
            else:
                # this is only a output event.
                # it's orphan, I don't know who action is broadcasting them.
                # TODO: analize the function source code and search
                # the output-events: broadcast, post_event, send_event
                pass

        return points

    def get_next_state_points(self):
        points = []
        for state in self.states:
            for name, values  in iteritems_(state.output_points):
                output_point, connector, next_state = values
                if not next_state:
                    continue
                point = Point(connector.attribs['x'], connector.attribs['y'])
                point = point.shift(state.origin.x, state.origin.y)
                point = point.shift(self.origin.x, self.origin.y)
                point = point.shift(4, 2)
                point_next_state = self.get_state_point(next_state)
                points.append((point, point_next_state))
        return points

    def get_state_point(self, state_name):
        for state in self.states:
            if state.name == state_name:
                return state.origin.shift(
                    self.origin.x + self.cx,
                    self.origin.y + self.layer.metrics.ascent
                )

    @property
    def size(self):
        return Size(self.cx, self.cy)


class InputEventLine(Group):
    def __init__(self, layer, name, start_point, end_points, color, level):
        super(InputEventLine, self).__init__(
            fill='none',
            stroke=color,
            stroke_width=layer.metrics.event_line_width,
            class_=name,
        )
        self.layer = layer
        self.name = name
        self.level = level
        self.start_point = start_point
        self.end_points = end_points
        self.build()
        highlight = svgwrite.animate.Set(
            attributeName='opacity',
            to='0.5',
            begin='mouseover',
            end='mouseout',
        )
        self.add(highlight)

    def add_right_arrow(self, points_list, end_point, size):
        points_list.append((end_point.x, end_point.y))
        points_list.append((end_point.x - size, end_point.y - size))
        points_list.append((end_point.x - size, end_point.y + size))
        points_list.append((end_point.x, end_point.y))

    def line_rect(self, start, end, level):
        r = 20
        d = [('M', start.x, start.y)]
        if level == 'top':
            d.append(('L', start.x, end.y - r))  # middle point
            d.append(('Q', start.x, end.y, start.x + r, end.y))
            d.append(('L', end.x, end.y))
        else:
            d.append(('L', start.x, end.y + r))  # middle point
            d.append(('Q', start.x, end.y, start.x + r, end.y))
            d.append(('L', end.x, end.y))
        line = svgwrite.path.Path(d=d)
        self.add(line)
        right_arrow = svgwrite.path.Path(d=right_arrow_path(end.x, end.y, 4))
        self.add(right_arrow)

    def build(self):
        for end_point in self.end_points:
            self.line_rect(self.start_point, end_point, self.level)


class OutputEventLine(Group):
    def __init__(self, layer, name, start_points, end_point, color, level):
        super(OutputEventLine, self).__init__(
            fill='none',
            stroke=color,
            stroke_width=layer.metrics.event_line_width,
            class_=name,
        )
        self.layer = layer
        self.name = name
        self.level = level
        self.start_points = start_points
        self.end_point = end_point
        self.build()
        highlight = svgwrite.animate.Set(
            attributeName='opacity',
            to='0.5',
            begin='mouseover',
            end='mouseout',
        )
        self.add(highlight)

    def add_top_arrow(self, points_list, end_point, size):
        points_list.append((end_point.x, end_point.y))
        points_list.append((end_point.x - size, end_point.y + size))
        points_list.append((end_point.x + size, end_point.y + size))
        points_list.append((end_point.x, end_point.y))

    def add_bottom_arrow(self, points_list, end_point, size):
        points_list.append((end_point.x, end_point.y))
        points_list.append((end_point.x + size, end_point.y - size))
        points_list.append((end_point.x - size, end_point.y - size))
        points_list.append((end_point.x, end_point.y))

    def line_rect(self, start, end, level):
        r = 20
        d = [('M', start.x, start.y)]
        if level == 'top':
            if start.x != end.x:
                d.append(('L', end.x - r, start.y))  # middle point
                d.append(('Q', end.x, start.y, end.x, start.y - r))
            d.append(('L', end.x, end.y))
        else:
            if start.x != end.x:
                d.append(('L', end.x - r, start.y))  # middle point
                d.append(('Q', end.x, start.y, end.x, start.y + r))
            d.append(('L', end.x, end.y))
        line = svgwrite.path.Path(d=d)
        self.add(line)

        if level == 'top':
            arrow = svgwrite.path.Path(d=top_arrow_path(end.x, end.y, 4))
        else:
            arrow = svgwrite.path.Path(d=bottom_arrow_path(end.x, end.y, 4))

        self.add(arrow)

    def build(self):
        for start_point in self.start_points:
            self.line_rect(start_point, self.end_point, self.level)


class StateChangeCurve(Group):
    def __init__(self, layer, name, start_point, end_point, axis, color):
        super(StateChangeCurve, self).__init__(
            fill='none',
            stroke=color,
            stroke_width=layer.metrics.state_change_line_width,
        )
        self.layer = layer
        self.name = name
        self.start_point = start_point
        self.end_point = end_point
        self.axis = axis
        self.build()
        highlight = svgwrite.animate.Set(
            attributeName='opacity',
            to='0.5',
            begin='mouseover',
            end='mouseout',
        )
        self.add(highlight)

    def build(self):
        self.line_curve(self.start_point, self.end_point, self.axis)

    def line_curve(self, start, end, axis):
        r = 10
        d = [('M', start.x, start.y)]
        if end.y > start.y:
            # to down
            d.append(('L', axis - r, start.y))
            d.append(('Q', axis, start.y, axis, start.y + r))
            d.append(('L', axis, end.y - r))
            d.append(('Q', axis, end.y, axis - r, end.y))
            d.append(('L', end.x, end.y))
        else:
            # to up
            d.append(('L', axis - r, start.y))
            d.append(('Q', axis, start.y, axis, start.y - r))
            d.append(('L', axis, end.y + r))
            d.append(('Q', axis, end.y, axis - r, end.y))
            d.append(('L', end.x, end.y))
        line = svgwrite.path.Path(d=d)
        self.add(line)

        arrow = svgwrite.path.Path(
            d=left_arrow_path(end.x, end.y, 8),
            fill='black',
        )

        self.add(arrow)


class Layer(Group):
    """
        Three zones: event zone, state zone and new-state zone:

          ev-zone   st-zone       ns-zone
        +---------+--------------+---------+
        |                                  |
        |           event names            |
        |                                  |
        +---------+--------------+---------+ -> tt_axis
        ||--------+--------------+--------||
        ||        |              |        ||
        ||        |   sep zone   |        ||
        ||        |              |        ||
        +---------+--------------+---------+ -> t_axis
        ||        |              |        ||
        ||        |              |        ||
        ||        |              |        ||
        ||        |   states     |        ||
        ||        |              |        ||
        ||        |              |        ||
        ||        |              |        ||
        +---------+--------------+---------+ -> b_axis
        ||        |              |        ||
        ||        |   sep zone   |        ||
        ||        |              |        ||
        ||--------|--------------|--------||
        +---------+--------------+---------+ -> bb_axis
        |                                  |
        |            event names           |
        |                                  |
        +---------+--------------+---------+
                  |              ^ r_axis
                  ^ l_axis

    """
    def __init__(self, smachine, font_name, font_size):
        super(Layer, self).__init__(font_family="DejaVu Sans",
                                    font_size=font_size)
        self.font_name = font_name
        self.font_size = font_size
        self.smachine = smachine
        self.metrics = Metrics(font_name, font_size)
        self.colors = {}

        #
        #   Build states
        #
        states = States(self, smachine)
        states_cx, states_cy = states.size

        top_input_events = smachine.get_top_input_events()
        top_output_events = smachine.get_top_output_events()
        bottom_input_events = smachine.get_bottom_input_events()
        bottom_output_events = smachine.get_bottom_output_events()
        state_changes = len(states.get_next_state_points()) + 1

        top_margin = self.metrics.top_margin
        bottom_margin = self.metrics.bottom_margin
        left_margin = self.metrics.left_margin
        right_margin = self.metrics.right_margin
        ev_line_sep = self.metrics.ev_line_sep
        st_line_sep = self.metrics.st_line_sep

        # let top space for top event_names
        if len(top_input_events) or len(top_output_events):
            self.tt_axis = self.metrics.tt_axis
        else:
            self.tt_axis = 20
        self.tt_axis += top_margin

        self.t_axis = self.metrics.t_axis
        self.t_axis += self.tt_axis

        # let left-right space for top/bottom input event lines
        self.l_axis = len(top_input_events) * ev_line_sep + \
                        len(bottom_input_events) * ev_line_sep + \
                        ev_line_sep + left_margin
        self.r_axis = self.l_axis + states_cx
        states.translate(self.l_axis, self.t_axis)

        # let left-right space for top/bottom output event lines
        self.cx = self.l_axis + \
                    states_cx + \
                    len(top_output_events) * ev_line_sep + \
                    len(bottom_output_events) * ev_line_sep + \
                    state_changes * st_line_sep + \
                    ev_line_sep + right_margin

        # let bottom space for bottom event_names
        self.b_axis = states_cy
        self.b_axis += self.t_axis
        self.bb_axis = self.metrics.bb_axis
        self.bb_axis += self.b_axis

        # extend height

        if len(bottom_input_events) or len(bottom_output_events):
            self.cy = self.bb_axis + self.metrics.tt_axis + bottom_margin
        else:
            self.cy = self.bb_axis + bottom_margin

        # assign color to events
        events = self.smachine.get_event_list()
        for idx, ev_name in enumerate(events):
            self.colors[ev_name] = colors[idx]

        # draw smachine rectangle
        smachine_rect = svgwrite.shapes.Rect(
            insert=(left_margin, self.tt_axis),
            size=(self.cx - left_margin - right_margin,
                  self.bb_axis - self.tt_axis),
            rx=10,
            ry=10,
            fill='#f8f8f8',
            stroke='#888a85',
            stroke_width=2,
        )
        self.add(smachine_rect)

        # draw states
        self.add(states)

        # draw smachine name
        self.add(svgwrite.text.Text(
            smachine.name,
            insert=(self.l_axis, top_margin),
            font_size=font_size * 2,
        ))

        # draw top input event lines
        gidx = 1
        for idx, ev_name in enumerate(top_input_events):
            start_point = Point(
                gidx * ev_line_sep + left_margin,
                self.tt_axis - 20
            )
            end_points = states.get_input_points(ev_name)
            line = InputEventLine(
                self,
                ev_name,
                start_point,
                end_points,
                self.colors[ev_name],
                level='top')
            self.add(line)

            text = svgwrite.text.Text(
                ev_name,
                insert=start_point.shift(0, -4),
            )
            text.rotate(-45, start_point)
            self.add(text)
            gidx += 1

        # draw bottom input event lines
        for idx, ev_name in enumerate(bottom_input_events):
            #drawing.defs.add(drawing.style('stylesheet-content'))

            start_point = Point(
                gidx * ev_line_sep + left_margin,
                self.bb_axis + 20
            )
            end_points = states.get_input_points(ev_name)
            line = InputEventLine(
                self,
                ev_name,
                start_point,
                end_points,
                self.colors[ev_name],
                level='bottom')
            self.add(line)

            text = svgwrite.text.Text(
                ev_name,
                insert=start_point.shift(0, self.metrics.ascent),
            )
            text.rotate(45, start_point)
            self.add(text)
            gidx += 1

        # draw top output event lines
        gidx = 1
        for idx, ev_name in enumerate(top_output_events):
            end_point = Point(
                gidx * ev_line_sep + self.r_axis + state_changes * st_line_sep,
                self.tt_axis - 20
            )
            states_filter = smachine.get_top_output_event_states(ev_name)
            start_points = states.get_output_points(ev_name, states_filter)
            if not start_points:
                # only output event
                start_points = [end_point.shift(0, 40)]
            line = OutputEventLine(
                self,
                ev_name,
                start_points,
                end_point,
                self.colors[ev_name],
                level='top')
            self.add(line)

            text = svgwrite.text.Text(
                ev_name,
                insert=end_point.shift(2, -4),
            )
            text.rotate(-45, end_point)
            self.add(text)
            gidx += 1

        # draw bottom output event lines
        for idx, ev_name in enumerate(bottom_output_events):
            end_point = Point(
                gidx * ev_line_sep + self.r_axis + state_changes * st_line_sep,
                self.bb_axis + 20
            )
            states_filter = smachine.get_bottom_output_event_states(ev_name)
            start_points = states.get_output_points(ev_name, states_filter)
            if not start_points:
                # only output event
                start_points = [end_point.shift(0, -40)]
            line = OutputEventLine(
                self,
                ev_name,
                start_points,
                end_point,
                self.colors[ev_name],
                level='bottom')
            self.add(line)

            text = svgwrite.text.Text(
                ev_name,
                insert=end_point.shift(4, 8),
            )
            text.rotate(45, end_point)
            self.add(text)
            gidx += 1

        # draw state changes
        gidx = 2
        state_change_start_end_points = states.get_next_state_points()
        for start_end in state_change_start_end_points:
            line = StateChangeCurve(
                self,
                ev_name,
                start_end[0],
                start_end[1],
                gidx * st_line_sep + self.r_axis,
                'black')
            self.add(line)
            gidx += 1

        # draw a general rectangle
        general_rect = svgwrite.shapes.Rect(
            Point(1, 1),
            Size(self.cx - 2, self.cy - 2),
            rx=10,
            ry=10,
            fill='none',
            stroke='#000000',
            stroke_opacity=1,
            stroke_width=1,
            stroke_miterlimit=4,
            stroke_dasharray=4.4,
            stroke_dashoffset=0.2,
        )
        self.add(general_rect)

    @property
    def size(self):
        return Size(self.cx, self.cy)

    @property
    def width(self):
        return self.cx

    @property
    def height(self):
        return self.cy



# -*- coding: utf-8 -*-
""" Porting of DL_LIST.C
"""
import logging


class ListItem(object):
    def __init__(self):
        self.next = None
        self.prev = None


class DoubleList(object):
    def __init__(self):
        self.head = None
        self.tail = None
        self.itemsInContainer = 0


def dl_first(dl):
    return dl.head


def dl_last(dl):
    return dl.tail


def dl_next(it):
    return it.next


def dl_prev(it):
    return it.prev


def dl_flush(dl):
    """ Flush double list. (Remove all items).
    """
    curr = dl.head

    while not curr:
        temp = curr
        curr = curr.next
        del temp

    dl.head = None
    dl.tail = None
    dl.itemsInContainer = 0


def check_links(item):
    if item.prev or item.next:
        logging.error("ERROR dl_list.py: INSERTING ITEM WITH PREVIOUS LINKS")
        return False
    return True


def dl_add(dl, item):
    """ Add at end of the list (old dl_insert_tail)
    """
    if not check_links(item):
        return False

    if dl.tail is None:
        #---- Empty List -----
        item.prev = None
        item.next = None
        dl.head = item
        dl.tail = item

    else:
        #---- Last item ----
        item.prev = dl.tail
        item.next = None
        dl.tail.next = item
        dl.tail = item

    dl.itemsInContainer += 1
    return True


def dl_insert(dl, item):
    """ Insert at the head (old dl_insert_head)
    """
    if not check_links(item):
        return False

    if dl.head is None:
        #---- Empty List -----
        item.prev = None
        item.next = None
        dl.head = item
        dl.tail = item
    else:
        #---- First item -----
        item.prev = None
        item.next = dl.head
        dl.head.prev = item
        dl.head = item

    dl.itemsInContainer += 1
    return True


def dl_insert_before(dl, curr, new_item):
    """ Insert a new item before current item.
    """
    if not check_links(new_item):
        return False

    if dl.head is None:
        #---- Empty List -----
        new_item.prev = None
        new_item.next = None
        dl.head = new_item
        dl.tail = new_item
    elif curr is None or curr.prev is None:
        # First item. (curr==dl.head)
        new_item.prev = None
        new_item.next = dl.head
        dl.head.prev = new_item
        dl.head = new_item
    else:
        # Middle or last item
        curr.prev.next = new_item
        new_item.next = curr
        new_item.prev = curr.prev
        curr.prev = new_item

    dl.itemsInContainer += 1
    return True


def dl_insert_after(dl, curr, new_item):
    """ Insert a new item after current item.
    """
    if not check_links(new_item):
        return False

    if dl.head is None:
        #---- Empty List -----
        new_item.prev = None
        new_item.next = None
        dl.head = new_item
        dl.tail = new_item
    elif curr is None or curr.next is None:
        # Last item
        new_item.prev = dl.tail
        new_item.next = None
        dl.tail.next = new_item
        dl.tail = new_item
    else:
        curr.next.prev = new_item
        new_item.next = curr.next
        new_item.prev = curr
        curr.next = new_item

    dl.itemsInContainer += 1
    return True


def dl_delete(dl, curr):
    """ Delete current item
    """
    if curr is None:
        logging.error("ERROR dl_delete(): DELETING ITEM NULL")
        return False

    #
    #     Check list
    #
    if dl.head is None or dl.tail is None or dl.itemsInContainer == 0:
        logging.error("ERROR dl_delete(): DELETING ITEM IN EMPTY LIST")
        return False

    #
    #                 Delete
    #
    if curr.prev is None:
        # First item. (curr==dl.head)
        dl.head = dl.head.next
        if dl.head:  # is last item?
            dl.head.prev = None  # no
        else:
            dl.tail = None  # yes
    else:
        #   Middle or last item
        curr.prev.next = curr.next
        if curr.next:  # last?
            curr.next.prev = curr.prev  # no
        else:
            dl.tail = curr.prev  # yes

    #
    #  Decrement items counter
    #
    dl.itemsInContainer -= 1

    #
    #  Reset pointers
    #
    curr.prev = None
    curr.next = None

    #
    #  Free item
    #
    del curr

    return True


def dl_index(dl, item):
    """ Return list position of item (relative to 1)
        Return 0 if not found
    """
    curr = dl.head
    i = 0
    while not curr:
        if curr == item:
            return i + 1
        curr = curr.next
        i += 1
    return 0  # not found


def dl_size(dl):
    """ Return number of items in list
    """
    return dl.itemsInContainer



"""
Sample WSGI server
==================

It uses :class:`ginsfsm.wsgi.c_wsgi_server.GWSGIServer`.

.. autofunction:: main

"""

from ginsfsm.gaplic import GAplic
from ginsfsm.wsgi.c_wsgi_server import GWSGIServer


#===============================================================
#                   Server
#===============================================================
def application(environ, start_response):
    start_response('200 OK', [('Content-Type', 'text/plain')])
    yield 'Hello World\n'


def paste_app_factory(global_config, **local_conf):
    return application


#===============================================================
#                   Main
#===============================================================
def main(global_config, **settings):
    """ Entry point to run with gserve (PasteDeploy)
    """
    ga = GAplic('Example6', **settings)
    ga.create_gobj(
        'wsgi-server',
        GWSGIServer,
        None,
        host='0.0.0.0',
        port=8000,
        application=application
    )
    return ga


if __name__ == "__main__":
    ga = main({})

    try:
        ga.mt_process()
    except (KeyboardInterrupt, SystemExit):
        print('Program stopped')
